=== modified file 'lib/canonical/launchpad/browser/securitycontact.py'
--- lib/canonical/launchpad/browser/securitycontact.py	2006-06-05 22:19:47 +0000
+++ lib/canonical/launchpad/browser/securitycontact.py	2006-08-02 14:49:03 +0000
@@ -5,21 +5,29 @@
 __metaclass__ = type
 __all__ = ["SecurityContactEditView"]
 
+from zope.formlib.form import action
+
+from canonical.launchpad.interfaces import IHasSecurityContact
 from canonical.launchpad.webapp import canonical_url
-from canonical.launchpad.webapp.generalform import GeneralFormView
+from canonical.launchpad.webapp.launchpadform import LaunchpadFormView
 
-class SecurityContactEditView(GeneralFormView):
+class SecurityContactEditView(LaunchpadFormView):
     """Browser view for editing the security contact.
 
     self.context is assumed to implement IHasSecurityContact.
     """
 
+    schema = IHasSecurityContact
+    field_names = ['security_contact']
+
     @property
     def initial_values(self):
         return {
             'security_contact': self.context.security_contact}
 
-    def process(self, security_contact):
+    @action(u'Submit')
+    def handle_submit(self, action, data):
+        security_contact = data['security_contact']
         if self.context.security_contact == security_contact:
             return
 
@@ -44,5 +52,6 @@
             self.request.response.addNotification(
                 "Successfully removed the security contact")
 
-    def nextURL(self):
+    @property
+    def next_url(self):
         return canonical_url(self.context)

=== added file 'lib/canonical/launchpad/templates/launchpad-form.pt'
--- lib/canonical/launchpad/templates/launchpad-form.pt	1970-01-01 00:00:00 +0000
+++ lib/canonical/launchpad/templates/launchpad-form.pt	2006-08-02 14:49:03 +0000
@@ -0,0 +1,84 @@
+<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en"
+      lang="en"
+      metal:use-macro="context/@@main_template/master"
+      i18n:domain="launchpad">
+
+<body>
+
+<div metal:fill-slot="main">
+
+  <div tal:condition="nothing">
+    This is a general form. You should not edit this file, or copy it and
+    use it, but rather copy the example page template in
+    template-generalform.pt to your own form page, and edit that.
+  </div>
+
+  <div metal:define-macro="form">
+
+    <form action="." 
+          tal:attributes="action request/URL" 
+          method="post"
+          enctype="multipart/form-data">
+
+      <div metal:define-macro="formbody">
+
+        <h1 tal:condition="view/label"
+            tal:content="view/label"
+            metal:define-slot="heading"
+            >Add Something</h1>
+
+        <p metal:define-slot="extra_info" tal:replace="nothing">
+          This is the description of the form.
+          This text will be in bold, beneath the title of the add form,
+          and should give any introductory information required.
+          Don't use it to state the bleeding obvious!
+        </p>
+
+	<p class="error message"
+           tal:condition="view/errors"
+           tal:content="view/error_count" />
+
+        <p class="error message"
+          tal:repeat="top_of_page_error view/top_of_page_errors"
+          tal:content="structure top_of_page_error">
+          Schema validation errors.
+        </p>
+
+        <div class="row" 
+             metal:define-slot="extra_top" 
+             tal:replace="nothing">
+            <div>Extra top</div>
+            <div><input type="text" style="width:100%" /></div>
+        </div>
+
+        <metal:widgets metal:define-slot="widgets">
+        <div metal:use-macro="context/@@launchpad_widget_macros/launchpad_widget_rows" />
+        </metal:widgets>
+
+        <div class="row"
+             metal:define-slot="extra_bottom" 
+             tal:replace="nothing">
+          <div>Extra bottom</div>
+          <div class="field"><input type="text" style="width:100%" /></div>
+        </div>
+
+      </div>
+      <div class="actions" metal:define-slot="buttons">
+        <input tal:repeat="action view/actions"
+               tal:replace="structure action/render"
+          />
+      </div>
+
+      <div class="row" 
+           metal:define-slot="extra_buttons" 
+           tal:replace="nothing">
+      </div>
+
+    </form>
+  </div>
+
+  </div>
+  </body>
+
+</html>
+

=== modified file 'lib/canonical/launchpad/templates/securitycontact-edit.pt'
--- lib/canonical/launchpad/templates/securitycontact-edit.pt	2006-04-13 20:46:13 +0000
+++ lib/canonical/launchpad/templates/securitycontact-edit.pt	2006-08-02 14:49:03 +0000
@@ -20,7 +20,7 @@
 </metal:heading>
 
 <div metal:fill-slot="main">
-  <div metal:use-macro="context/@@launchpad_generalform/generalform">
+  <div metal:use-macro="context/@@launchpad_form/form">
 
     <p metal:fill-slot="extra_info">
       When a bug is reported as a security vulnerability, the security

=== added file 'lib/canonical/launchpad/templates/template-form.pt'
--- lib/canonical/launchpad/templates/template-form.pt	1970-01-01 00:00:00 +0000
+++ lib/canonical/launchpad/templates/template-form.pt	2006-08-02 14:49:03 +0000
@@ -0,0 +1,71 @@
+<html
+  xmlns="http://www.w3.org/1999/xhtml"
+  xml:lang="en"
+  lang="en"
+  metal:use-macro="context/@@main_template/master"
+  i18n:domain="launchpad">
+
+<body>
+
+<div tal:replace="nothing">
+  This page uses the launchpad-form.pt METAL. Copy this page to your
+  object-form.pt and then tweak the various bits below to make it all look
+  nice.
+</div>
+
+<metal:leftportlets fill-slot="portlets_one">
+</metal:leftportlets>
+
+<metal:rightportlets fill-slot="portlets_two">
+  <div tal:replace="structure context/@@+portlet-details" />
+</metal:rightportlets>
+
+<div metal:fill-slot="main">
+
+  <div metal:use-macro="context/@@launchpad_form/form">
+
+    <h1 tal:condition="view/label"
+        tal:content="view/label"
+        metal:fill-slot="heading">Heading from ZCML label="xxx"</h1>
+
+    <div metal:fill-slot="extra_info" class="documentDescription">
+      Please the introductory paragraph here. It will be displayed in
+      bold text below the form title. It should be a single paragraph
+      giving an overview of what is being added.
+    </div>
+
+    <div class="row" metal:fill-slot="extra_top" tal:replace="nothing">
+      <div>Extra top</div>
+      <div class="field"><input type="text" style="width:100%" /></div>
+      <div>
+        Put any extra form elements you want for this form which
+        are not autogenerated here, and they will be at the top of the
+        form. The auto-generated elements will use the layout from
+        launchpad-widget-macros.pt so try to follow that pattern.
+      </div>
+    </div>
+
+    <div class="row" metal:fill-slot="extra_bottom" tal:replace="nothing">
+      <div>Extra bottom</div>
+      <div class="field"><input type="text" style="width:100%" /></div>
+      <div>
+        If you want additional non-autogenerated widgets at the bottom of the
+        form, add them here.
+      </div>
+    </div>
+
+    <div class="row" metal:fill-slot="extra_buttons" tal:replace="nothing">
+      If you wish to add some more buttons to the form, put those here.
+    </div>
+
+  </div>
+
+  <div tal:replace="nothing">
+    This piece will appear at the bottom of the form if you want to add
+    a footer.
+  </div>
+
+</div>
+    
+</body>
+</html>

=== modified file 'lib/canonical/launchpad/webapp/launchpadform.py'
--- lib/canonical/launchpad/webapp/launchpadform.py	2006-08-02 13:59:01 +0000
+++ lib/canonical/launchpad/webapp/launchpadform.py	2006-08-02 14:49:03 +0000
@@ -41,7 +41,10 @@
     # the next URL to redirect to on successful form submission
     next_url = None
 
+    label = ''
+
     errors = ()
+    top_of_page_errors = ()
     actions = ()
 
     def initialize(self):
@@ -57,7 +60,7 @@
             return
             
         data = {}
-        errors, actions = handleSubmit(self.actions, data, self._validate)
+        errors, action = form.handleSubmit(self.actions, data, self._validate)
 
         if errors:
             action.failure(data, errors)
@@ -79,7 +82,7 @@
         transaction.abort()
 
     def setUpFields(self):
-        assert schema is not None, "Schema must be set for LaunchpadFormView"
+        assert self.schema is not None, "Schema must be set for LaunchpadFormView"
         # XXX: 20060802 jamesh
         # expose omit_readonly=True ??
         self.form_fields = form.Fields(self.schema)
@@ -91,7 +94,7 @@
         # do we want to do anything with ignore_request?
         self.widgets = form.setUpWidgets(
             self.form_fields, self.prefix, self.context, self.request,
-            data=initial_values, ignore_request=False)
+            data=self.initial_values, ignore_request=False)
 
     @property
     def initial_values(self):
@@ -132,3 +135,13 @@
         """
         pass
 
+    @property
+    def error_count(self):
+        # XXX: 20060802 jamesh
+        # this should use ngettext if we ever translate Launchpad's UI
+        if len(self.errors) == 0:
+            return ''
+        elif len(self.errors) == 1:
+            return 'There is 1 error'
+        else:
+            return 'There are %d errors' % len(self.errors)

=== modified file 'lib/canonical/launchpad/zcml/securitycontact.zcml'
--- lib/canonical/launchpad/zcml/securitycontact.zcml	2006-04-06 14:30:34 +0000
+++ lib/canonical/launchpad/zcml/securitycontact.zcml	2006-08-02 14:49:03 +0000
@@ -4,15 +4,12 @@
     xmlns:lp="http://launchpad.canonical.com"
     xmlns:i18n="http://namespaces.zope.org/i18n"
     i18n_domain="launchpad">
-    <browser:generalform
+    <browser:page
       name="+securitycontact"
       for="canonical.launchpad.interfaces.IHasSecurityContact"
-      schema="canonical.launchpad.interfaces.IHasSecurityContact"
       class="canonical.launchpad.browser.securitycontact.SecurityContactEditView"
       facet="bugs"
       permission="launchpad.Edit"
-      fields="security_contact"
-      keyword_arguments="security_contact"
       template="../templates/securitycontact-edit.pt"
       />
-</configure>
\ No newline at end of file
+</configure>

=== modified file 'lib/canonical/launchpad/zcml/widgets.zcml'
--- lib/canonical/launchpad/zcml/widgets.zcml	2005-10-31 18:29:12 +0000
+++ lib/canonical/launchpad/zcml/widgets.zcml	2006-08-02 14:49:03 +0000
@@ -40,6 +40,14 @@
         template="../templates/launchpad-editform.pt"
         />
 
+    <browser:page
+        for="*"
+        name="launchpad_form"
+        layer="canonical.launchpad.layers.LaunchpadLayer"
+        permission="zope.Public"
+        template="../templates/launchpad-form.pt"
+        />
+
     <!-- Widget Macros
          NB this file overrides the addform / editform widget
          display layout used in Launchpad. It should be somewhere

=== added file 'package-includes/formlib-configure.zcml'
--- package-includes/formlib-configure.zcml	1970-01-01 00:00:00 +0000
+++ package-includes/formlib-configure.zcml	2006-08-02 14:49:03 +0000
@@ -0,0 +1,9 @@
+<configure
+    xmlns="http://namespaces.zope.org/zope"
+    xmlns:i18n="http://namespaces.zope.org/i18n"
+    i18n_domain="zope"
+    >
+
+  <include package="zope.formlib" />
+
+</configure>

